﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TourCompanyContracts.BindingModels;
using TourCompanyContracts.BusinessLogicsContracts;
using TourCompanyContracts.SearchModels;
using TourCompanyContracts.StoragesContracts;
using TourCompanyContracts.ViewModels;

namespace TourCompanyBusinessLogic.BusinessLogics
{
    internal class GidLogic : IGidLogic
    {
        private readonly ILogger _logger;
        private readonly IGidStorage _gidStorage;
        public GidLogic(ILogger<GidLogic> logger, IGidStorage gidStorage)
        {
            _logger = logger;
            _gidStorage = gidStorage;
        }
        public List<GidViewModel>? ReadList(GidSearchModel? model)
        {
            _logger.LogInformation("ReadList. UserId:{UserId}. ExecurtionId:{ExecurtionId}. TripId:{TripId}. Id: { Id}", model?.UserId, model?.ExecurtionId, model?.TripId, model?.Id);
            var list = model == null ? _gidStorage.GetFullList() : _gidStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public GidViewModel? ReadElement(GidSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement.UserId:{UserId}. ExecurtionId:{ExecurtionId}. TripId:{TripId}. Id: { Id}", model.UserId, model.ExecurtionId, model.TripId, model.Id);
            var element = _gidStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool Create(GidBindingModel model)
        {
            CheckModel(model);
            if (_gidStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(GidBindingModel model)
        {
            CheckModel(model);
            if (_gidStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(GidBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_gidStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(GidBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.TripId == null)
            {
                throw new ArgumentNullException("нет внешнего ключа поездки", nameof(model.TripId));
            }
            if (model.UserId == null)
            {
                throw new ArgumentNullException("нет внешнего ключа пользователя", nameof(model.UserId));
            }
            if (model.ExecurtionId == null)
            {
                throw new ArgumentNullException("нет внешнего ключа экскурсии", nameof(model.ExecurtionId));
            }
            _logger.LogInformation("Gid. UserId:{UserId}. ExecurtionId:{ExecurtionId}. TripId:{TripId}. Id: { Id}", model.UserId, model.ExecurtionId, model.TripId, model.Id);
            var element = _gidStorage.GetElement(new GidSearchModel
            {
                TripId = model.TripId,
                UserId = model.UserId,
                ExecurtionId = model.ExecurtionId

            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Компонент с таким названием уже есть");
            }
        }
    }
}
